=== modified file 'baz2bzr'
--- baz2bzr	2005-06-03 19:03:46 +0000
+++ baz2bzr	2005-06-03 21:19:31 +0000
@@ -172,6 +172,41 @@
     ancestors.reverse()
     return ancestors
 
+def get_remaining_revisions(output_dir, version):
+    last_patch = None
+    old_revno = None
+    if os.path.exists(output_dir):
+        # We are starting from an existing directory, figure out what
+        # the current version is
+        branch = bzrlib.Branch(output_dir)
+        last_patch = branch.last_patch()
+        try:
+            last_patch = arch_revision(last_patch)
+        except NotArchRevision:
+            raise UserError(
+                "Directory \"%s\" already exists, and the last revision is not"
+                " an Arch revision (%s)" % (output_dir, last_patch))
+        if version is None:
+            version = last_patch.version
+    elif version is None:
+        raise UserError("No version specified, and directory does not exist.")
+
+    ancestors = version_ancestry(version)
+
+    if last_patch:
+        for i in range(len(ancestors)):
+            if ancestors[i] == last_patch:
+                break
+        else:
+            raise UserError("Directory \"%s\" already exists, and the last "
+                "revision (%s) is not in the ancestry of %s" % 
+                (output_dir, last_patch, version))
+        # Strip off all of the ancestors which are already present
+        # And get a directory starting with the latest ancestor
+        latest_ancestor = ancestors[i]
+        old_revno = bzrlib.Branch(output_dir).revno()
+        ancestors = ancestors[i+1:]
+    return ancestors, old_revno
 
 def import_version(output_dir, version, fancy=True, fast=False, verbose=False, 
                    dry_run=False, max_count=None, skip_symlinks=False):
@@ -187,19 +222,54 @@
     Import complete.
     >>> teardown_environ(q)
     """
+    ancestors, old_revno = get_remaining_revisions(output_dir, version)
+    if len(ancestors) == 0:
+        print '* Tree is up-to-date with %s' % last_patch
+        return 0
+
     progress_bar = ProgressBar()
     tempdir = tempfile.mkdtemp(prefix="baz2bzr-",
                                dir=os.path.dirname(output_dir))
     try:
         if not fancy:
             print "not fancy"
-        for result in iter_import_version(output_dir, version, tempdir,
+        for result in iter_import_version(output_dir, ancestors, tempdir,
                 fast=fast, verbose=verbose, dry_run=dry_run, 
                 max_count=max_count, skip_symlinks=skip_symlinks):
             if fancy:
                 progress_bar(result)
             else:
                 sys.stdout.write('.')
+
+        if dry_run:
+            print '**Dry run, not modifying output_dir'
+            return 0
+        if os.path.exists(output_dir):
+            # Move the bzr control directory back, and update the working tree
+            tmp_bzr_dir = os.path.join(tempdir, '.bzr')
+            
+            bzr_dir = os.path.join(output_dir, '.bzr')
+            new_bzr_dir = os.path.join(tempdir, "rd", '.bzr')
+
+            os.rename(bzr_dir, tmp_bzr_dir) # Move the original bzr out of the way
+            os.rename(new_bzr_dir, bzr_dir)
+            try:
+                # bzrlib.merge that exists in mainline does not have a this_dir component,
+                # so we have to work in the local directory
+                try:
+                    pwd = os.getcwd()
+                    os.chdir(output_dir)
+                    bzrlib.merge.merge(('.', -1), ('.', old_revno))
+                finally:
+                    os.chdir(pwd)
+            except:
+                # If something failed, move back the original bzr directory
+                os.rename(bzr_dir, new_bzr_dir)
+                os.rename(tmp_bzr_dir, bzr_dir)
+                raise
+        else:
+            os.rename(revdir, output_dir)
+
     finally:
         if fancy:
             clear_progress_bar()
@@ -251,53 +321,10 @@
         except pybaz.errors.NamespaceError, e:
             raise NotArchRevision(revision_id)
             
-def iter_import_version(output_dir, version, tempdir, fast=False,
+def iter_import_version(output_dir, ancestors, tempdir, fast=False,
                         verbose=False, dry_run=False, max_count=None,
                         skip_symlinks=False):
     revdir = None
-    ancestors = version_ancestry(version)
-    last_patch = None
-    if os.path.exists(output_dir):
-        # We are starting from an existing directory, figure out what
-        # the current version is
-        branch = bzrlib.Branch(output_dir)
-        last_patch = branch.last_patch()
-        try:
-            last_patch = arch_revision(last_patch)
-        except NotArchRevision:
-            raise UserError(
-                "Directory \"%s\" already exists, and the last patch is not"
-                " an Arch patch (%s)" % (output_dir, last_patch))
-
-    if last_patch:
-        for i in range(len(ancestors)):
-            if ancestors[i] == last_patch:
-                break
-        else:
-            raise UserError("Directory \"%s\" already exists, and the last patch (%s) is not in the revision history" % (output_dir, last_patch))
-        # Strip off all of the ancestors which are already present
-        # And get a directory starting with the latest ancestor
-        latest_ancestor = ancestors[i]
-        old_revno = bzrlib.Branch(output_dir).revno()
-        ancestors = ancestors[i+1:]
-
-        if len(ancestors) == 0:
-            print '* Tree is up-to-date with %s' % last_patch
-            return
-
-        revdir = os.path.join(tempdir, "rd")
-        baz_inv, log = get_revision(revdir, latest_ancestor,
-                                    skip_symlinks=skip_symlinks)
-        bzr_dir = os.path.join(output_dir, '.bzr')
-        new_bzr_dir = os.path.join(revdir, '.bzr')
-        # This would be much faster with a simple os.rename(), but if we fail,
-        # we have corrupted the original .bzr directory.
-        # Is that a big problem, as we can just back out the last revisions in
-        # .bzr/revision_history
-        # I don't really know
-        shutil.copytree(bzr_dir, new_bzr_dir)
-        # Now revdir should have a tree with the latest .bzr, and the correct
-        # version of the baz tree
 
     # Uncomment this for testing, it basically just has baz2bzr only update
     # 5 patches at a time
@@ -330,7 +357,19 @@
             revdir = os.path.join(tempdir, "rd")
             baz_inv, log = get_revision(revdir, revision, 
                                         skip_symlinks=skip_symlinks)
-            branch = bzrlib.Branch(revdir, init=True)
+            if os.path.exists(output_dir):
+                bzr_dir = os.path.join(output_dir, '.bzr')
+                new_bzr_dir = os.path.join(tempdir, "rd", '.bzr')
+                # This would be much faster with a simple os.rename(), but if
+                # we fail, we have corrupted the original .bzr directory.  Is
+                # that a big problem, as we can just back out the last
+                # revisions in .bzr/revision_history I don't really know
+                shutil.copytree(bzr_dir, new_bzr_dir)
+                # Now revdir should have a tree with the latest .bzr, and the
+                # next revision of the baz tree
+                branch = bzrlib.Branch(revdir)
+            else:
+                branch = bzrlib.Branch(revdir, init=True)
         else:
             old = os.path.join(revdir, ".bzr")
             new = os.path.join(tempdir, ".bzr")
@@ -352,30 +391,6 @@
             branch.unlock()
     yield Progress("revisions", len(ancestors), len(ancestors))
     unlink_unversioned(branch, revdir)
-    if dry_run:
-        print '**Dry run, not modifying output_dir'
-        return
-    if os.path.exists(output_dir):
-        # Move the bzr control directory back, and update the working tree
-        tmp_bzr_dir = os.path.join(tempdir, '.bzr')
-        os.rename(bzr_dir, tmp_bzr_dir) # Move the original bzr out of the way
-        os.rename(new_bzr_dir, bzr_dir)
-        try:
-            # bzrlib.merge that exists in mainline does not have a this_dir component,
-            # so we have to work in the local directory
-            try:
-                pwd = os.getcwd()
-                os.chdir(output_dir)
-                bzrlib.merge.merge(('.', -1), ('.', old_revno))
-            finally:
-                os.chdir(pwd)
-        except:
-            # If something failed, move back the original bzr directory
-            os.rename(bzr_dir, new_bzr_dir)
-            os.rename(tmp_bzr_dir, bzr_dir)
-            raise
-    else:
-        os.rename(revdir, output_dir)
 
 def unlink_unversioned(branch, revdir):
     for unversioned in branch.working_tree().extras():
@@ -453,7 +468,7 @@
     :param args: The arguments to this script. Essentially sys.argv[1:]
     """
     import optparse
-    parser = optparse.OptionParser(usage='%prog [options] VERSION OUTDIR'
+    parser = optparse.OptionParser(usage='%prog [options] [VERSION] OUTDIR'
         '\n  VERSION is the arch version to import.'
         '\n  OUTDIR can be an existing directory to be updated'
         '\n         or a new directory which will be created from scratch.')
@@ -493,24 +508,29 @@
             return 1
         else:
             return 0
-
-    if len(args) != 2:
+    if len(args) == 2:
+        output_dir = os.path.realpath(args[1]) 
+        version = pybaz.Version(args[0])
+    elif len(args) == 1:
+        output_dir = os.path.realpath(args[0])
+        version = None
+    else:
         print 'Invalid number of arguments, try --help for more info'
         return 1
-    else:
-        try:
-            output_dir = os.path.realpath(args[1])
-            import_version(output_dir, pybaz.Version(args[0]),
-                verbose=opts.verbose, fast=opts.fast,
-                fancy=opts.fancy, dry_run=opts.dry_run,
-                max_count=opts.max_count, skip_symlinks=opts.skip_symlinks)
-            return 0
-        except UserError, e:
-            print e
-            return 1
-        except KeyboardInterrupt:
-            print "Aborted."
-            return 1
+        
+    try:
+        
+        import_version(output_dir, version,
+            verbose=opts.verbose, fast=opts.fast,
+            fancy=opts.fancy, dry_run=opts.dry_run,
+            max_count=opts.max_count, skip_symlinks=opts.skip_symlinks)
+        return 0
+    except UserError, e:
+        print e
+        return 1
+    except KeyboardInterrupt:
+        print "Aborted."
+        return 1
 
 
 if __name__ == '__main__':

